"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateFormTemplate = void 0;
const schematics_1 = require("@angular-devkit/schematics");
const listFiles_1 = require("../utils/listFiles");
const core_1 = require("@angular-devkit/core");
const ts = require("@schematics/angular/third_party/github.com/Microsoft/TypeScript/lib/typescript");
const change_1 = require("@schematics/angular/utility/change");
const updateFile_1 = require("../utils/updateFile");
const utils_1 = require("../utils/utils");
class TemplateFormSchema {
    constructor() {
        this.envFilePath = utils_1.DevEnvFile;
        this.formsServiceFilePath = utils_1.FormsServiceFilePath;
    }
}
function generateFormTemplate(options) {
    return () => {
        return (0, schematics_1.chain)([
            updateClassFile(options)
        ]);
    };
}
exports.generateFormTemplate = generateFormTemplate;
let updateClassFile = (options) => {
    return (tree) => {
        let files = (0, listFiles_1.listFiles)(tree, options.path);
        let targetFiles = [
            { suffix: "component.ts", predicate: updateCpntFile },
            { suffix: "service.ts", predicate: updateCpntFile },
            { suffix: "directive.ts", predicate: updateCpntFile },
        ];
        let targetFileName = files.find((file) => {
            return targetFiles.some((val) => file.endsWith(val.suffix));
        });
        let targetFileInfo = targetFiles.find((val) => {
            return targetFileName.endsWith(val.suffix);
        });
        let targetFilePath = (0, core_1.normalize)(`${targetFileName}`);
        targetFileInfo === null || targetFileInfo === void 0 ? void 0 : targetFileInfo.predicate(options, tree, targetFilePath);
    };
};
function updateCpntFile(options, tree, targetFilePath) {
    var _a;
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    let cpntClassVar = getCpntClassVar(sourceFile);
    let cpntClassName = (_a = cpntClassVar.name) === null || _a === void 0 ? void 0 : _a.getText();
    if (!cpntClassName) {
        throw new schematics_1.SchematicsException("component class name is missing or misspelt or the script is having issues finding the component class name");
    }
    let lowerCamelCaseCpntClassName = schematics_1.strings.camelize(cpntClassName.split("Component")[0]);
    let cpntClassNameWOCpnt = cpntClassName.split("Component")[0];
    let lastProperty = cpntClassVar.members
        .filter(ts.isPropertyDeclaration)
        .at(-1);
    injectSnippet(options, lowerCamelCaseCpntClassName, lastProperty, targetFilePath, tree, cpntClassNameWOCpnt);
    addInitFormCallToNgOnInit(targetFilePath, tree);
    addFormsServiceToConstructor(targetFilePath, tree);
    addImportStatements(sourceFile, targetFilePath, tree);
    updateEnvFile(tree, lowerCamelCaseCpntClassName, options);
    updateFormsServiceFile(tree, lowerCamelCaseCpntClassName, options);
    // sourceFile = wmlCreateSourceFile(targetFilePath, tree);
    // console.log(sourceFile.getFullText())
    // throw new SchematicsException("got here")
}
let updateFormsServiceFile = (tree, lowerCamelCaseCpntClassName, options) => {
    var _a;
    let { targetPath } = (0, utils_1.getFile)(tree, utils_1.FormsServiceFilePath);
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetPath, tree);
    let targetClass = (_a = sourceFile === null || sourceFile === void 0 ? void 0 : sourceFile.statements) === null || _a === void 0 ? void 0 : _a.filter(ts.isClassDeclaration).find((v) => {
        var _a;
        return ((_a = v.name) === null || _a === void 0 ? void 0 : _a.getText()) === 'FormsService';
    });
    let insertPos = 0;
    let snippet = "";
    let indentTarget;
    let spacingPrefix = "";
    let fieldsArray = options.fields.map((val) => {
        return `[ENV.${lowerCamelCaseCpntClassName}.${options.name}.${val}FormControlName]:new FormControl("")`;
    });
    let fieldsString = "_add_newline_here_\t" + fieldsArray.join(",_add_newline_here_\t") + "_add_newline_here_";
    let formBlock = `${options.name} :new FormGroup({${fieldsString}})`;
    let entityProp = (0, utils_1.wmlFindChildNode)(targetClass.members, 
    // @ts-ignore
    (prop) => {
        var _a;
        let name = (_a = prop.name) === null || _a === void 0 ? void 0 : _a.getText();
        return name === lowerCamelCaseCpntClassName;
    }, "its with the properties of FormsService", false, false);
    if (!entityProp) {
        let constructor = (0, utils_1.wmlFindChildNode)(targetClass.members, 
        // @ts-ignore
        ts.SyntaxKind.Constructor, "look like the constructor", false, false);
        snippet = `
    ${lowerCamelCaseCpntClassName}={
      ${formBlock}
    }
    `;
        spacingPrefix = "\n";
        if (!constructor) {
            let openBrace = (0, utils_1.wmlFindChildNode)(targetClass, ts.SyntaxKind.OpenBraceToken, "its with FormService Class open brace token is it missing");
            indentTarget = targetClass;
            insertPos = openBrace.end;
        }
        else {
            indentTarget = constructor;
            insertPos = constructor.end;
        }
    }
    else {
        fieldsString = "_add_newline_here_\t" + fieldsArray.join(",_add_newline_here_\t") + "_add_newline_here_\t";
        formBlock = `${options.name} :new FormGroup({${fieldsString}})`;
        let objLiteral = (0, utils_1.wmlFindChildNode)(entityProp, ts.SyntaxKind.ObjectLiteralExpression, "its with the braces on  the " + entityProp + " property", true);
        let closeBrace = (0, utils_1.wmlFindChildNode)(objLiteral, ts.SyntaxKind.CloseBraceToken, "its with the braces on  the " + entityProp + " property");
        indentTarget = closeBrace;
        insertPos = closeBrace.pos;
        let occurencess = objLiteral.properties.length;
        snippet = "\n" + formBlock;
        if (occurencess > 0) {
            let lastRouteLiteral = objLiteral.properties.at(-1);
            indentTarget = lastRouteLiteral;
            insertPos = lastRouteLiteral.end;
            spacingPrefix = ",";
        }
    }
    (0, utils_1.prepareForUpdate)(snippet, targetPath, tree, insertPos, 
    // @ts-ignore
    indentTarget === null || indentTarget === void 0 ? void 0 : indentTarget.getFullText(), spacingPrefix);
};
let updateEnvFile = (tree, lowerCamelCaseCpntClassName, options) => {
    var _a;
    let { targetPath } = (0, utils_1.getFile)(tree, utils_1.DevEnvFile);
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetPath, tree);
    let targetClass = (_a = sourceFile === null || sourceFile === void 0 ? void 0 : sourceFile.statements) === null || _a === void 0 ? void 0 : _a.filter(ts.isClassDeclaration).find((v) => {
        var _a;
        return ((_a = v.name) === null || _a === void 0 ? void 0 : _a.getText()) === 'DevEnv';
    });
    let insertPos = 0;
    let snippet = "";
    let indentTarget;
    let spacingPrefix = "";
    let fieldsArray = options.fields.map((val) => {
        return `${val + "FormControlName"}:"${val}"`;
    });
    let fieldsString = "{_add_newline_here_\t" + fieldsArray.join(",_add_newline_here_\t") + "_add_newline_here_}";
    let formBlock = `${options.name} :${fieldsString}
  `;
    let entityProp = (0, utils_1.wmlFindChildNode)(targetClass.members, 
    // @ts-ignore
    (prop) => {
        var _a;
        let name = (_a = prop.name) === null || _a === void 0 ? void 0 : _a.getText();
        return name === lowerCamelCaseCpntClassName;
    }, "its with the properties of DevENv", false, false);
    if (!entityProp) {
        let navVar = (0, utils_1.wmlFindChildNode)(targetClass.members, 
        // @ts-ignore
        (prop) => {
            var _a;
            return ((_a = prop.name) === null || _a === void 0 ? void 0 : _a.getText()) === 'nav';
        }, "look like the nav property is missing", false, false);
        snippet = `
    ${lowerCamelCaseCpntClassName}={
      ${formBlock}
    }
    `;
        if (!navVar) {
            let openBrace = (0, utils_1.wmlFindChildNode)(targetClass, ts.SyntaxKind.OpenBraceToken, "its with DevEnv classes open brace token is it missing");
            indentTarget = openBrace;
            insertPos = openBrace.end;
            spacingPrefix = "\n";
        }
        else {
            indentTarget = navVar;
            insertPos = navVar.end;
        }
    }
    else {
        let fieldsString = "{_add_newline_here_\t" + fieldsArray.join(",_add_newline_here_\t") + "_add_newline_here_\t}";
        let formBlock = `${options.name} :${fieldsString}
    `;
        let objLiteral = (0, utils_1.wmlFindChildNode)(entityProp, ts.SyntaxKind.ObjectLiteralExpression, "its with the braces on  the " + entityProp + " property", true);
        let closeBrace = (0, utils_1.wmlFindChildNode)(objLiteral, ts.SyntaxKind.CloseBraceToken, "its with the braces on  the " + entityProp + " property");
        indentTarget = closeBrace;
        insertPos = closeBrace.pos;
        let occurencess = objLiteral.properties.length;
        snippet = "\n" + formBlock;
        if (occurencess > 0) {
            let lastRouteLiteral = objLiteral.properties.at(-1);
            indentTarget = lastRouteLiteral;
            insertPos = lastRouteLiteral.end;
            spacingPrefix = ",";
        }
    }
    (0, utils_1.prepareForUpdate)(snippet, targetPath, tree, insertPos, indentTarget === null || indentTarget === void 0 ? void 0 : indentTarget.getFullText(), spacingPrefix);
};
function addImportStatements(sourceFile, targetFilePath, tree) {
    let importStatementSnippet = `\n`;
    sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    let statements = sourceFile.statements
        .filter(ts.isImportDeclaration);
    const importsToAdd = [
        {
            packageName: "@windmillcode/angular-wml-form",
            importStatement: "import { WMLForm } from '@windmillcode/angular-wml-form';\n",
            importMembers: ["WMLForm"]
        },
        {
            packageName: "@windmillcode/angular-wml-button-zero",
            importStatement: "import { WmlButtonZeroParams, WMLButtonParamsTypeEnum } from '@windmillcode/angular-wml-button-zero';\n",
            importMembers: ["WmlButtonZeroParams", "WMLButtonParamsTypeEnum"]
        },
        {
            packageName: "@windmillcode/angular-wml-components-base",
            importStatement: "import { WMLUIProperty, generateClassPrefix } from '@windmillcode/angular-wml-components-base';\n",
            importMembers: ["WMLUIProperty", "generateClassPrefix"]
        },
        {
            packageName: "@core/base/base.service",
            importStatement: "import { BaseService, BaseServiceSubmitFormParams } from '@core/base/base.service';\n",
            importMembers: ["BaseService", "BaseServiceSubmitFormParams"]
        },
        {
            packageName: "@core/utility/form-utils",
            importStatement: "import { createI18nErrorMsgsBasedOnValidations } from '@core/utility/form-utils';\n",
            importMembers: ["createI18nErrorMsgsBasedOnValidations"]
        },
        {
            packageName: "@shared/services/forms/forms.service",
            importStatement: "import { FormsService } from '@shared/services/forms/forms.service';\n",
            importMembers: ["FormsService"]
        }
    ];
    for (const importData of importsToAdd) {
        importStatementSnippet = (0, utils_1.addImportStatement)(targetFilePath, tree, importStatementSnippet, importData.packageName, importData.importStatement, importData.importMembers);
    }
    sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    statements = sourceFile.statements
        .filter(ts.isImportDeclaration);
    let lastMember = statements.at(-1);
    (0, utils_1.prepareForUpdate)(importStatementSnippet, targetFilePath, tree, lastMember === null || lastMember === void 0 ? void 0 : lastMember.end, lastMember === null || lastMember === void 0 ? void 0 : lastMember.getFullText());
}
function addInitFormCallToNgOnInit(targetFilePath, tree) {
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    let cpntClassVar = getCpntClassVar(sourceFile);
    let ngOnInit = cpntClassVar.members
        .filter((x) => {
        return x.kind === ts.SyntaxKind.MethodDeclaration;
    })
        .find((x) => {
        var _a;
        return ((_a = x.name) === null || _a === void 0 ? void 0 : _a.getText()) === "ngOnInit";
    });
    if (!ngOnInit) {
        let constructor = (0, utils_1.wmlFindChildNode)(cpntClassVar.members, ts.SyntaxKind.Constructor, "its with the constructor", false, false);
        let ngOnInitStr = `\n
    ngOnInit(): void{
      this.initForm() _add_newline_here_
    }
    `;
        if (!constructor) {
            let openBrace = (0, utils_1.wmlFindChildNode)(cpntClassVar, ts.SyntaxKind.OpenBraceToken, "its with component class close {", false, false);
            (0, utils_1.prepareForUpdate)(ngOnInitStr, targetFilePath, tree, openBrace.end, cpntClassVar.getFullText());
        }
        else {
            let openParenToken = (0, utils_1.wmlFindChildNode)(constructor, ts.SyntaxKind.OpenParenToken, "its with constuctor close parentheses", false, false);
            (0, utils_1.prepareForUpdate)(ngOnInitStr, targetFilePath, tree, openParenToken.end, constructor.getFullText());
        }
    }
    else {
        let block = (0, utils_1.wmlFindChildNode)(ngOnInit, ts.SyntaxKind.Block, "its with ngOnInit's block");
        let openBrace = (0, utils_1.wmlFindChildNode)(block, ts.SyntaxKind.OpenBraceToken, "its with ngOnInit's open brace");
        (0, utils_1.prepareForUpdate)(`\nthis.initForm() _add_newline_here_`, targetFilePath, tree, openBrace.end, ngOnInit.getFullText(), "\t\t");
    }
}
function addFormsServiceToConstructor(targetFilePath, tree) {
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    let cpntClassVar = getCpntClassVar(sourceFile);
    let constructor = (0, utils_1.wmlFindChildNode)(cpntClassVar.members, ts.SyntaxKind.Constructor, "its with the constructor", false, false);
    if (!constructor) {
        let openBrace = (0, utils_1.wmlFindChildNode)(cpntClassVar, ts.SyntaxKind.OpenBraceToken, "its with component class close {", false, false);
        (0, utils_1.prepareForUpdate)(` \n   constructor(
        public formsService:FormsService
      ) {}`, targetFilePath, tree, openBrace.end, cpntClassVar.getFullText());
    }
    else {
        let openParenToken = (0, utils_1.wmlFindChildNode)(constructor, ts.SyntaxKind.OpenParenToken, "its with constuctor close parentheses", false, false);
        (0, utils_1.prepareForUpdate)(`  public formsService:FormsService,`, targetFilePath, tree, openParenToken.end, constructor.getFullText());
    }
}
function injectSnippet(options, lowerCamelCaseCpntClassName, lastProperty, targetFilePath, tree, cpntClassNameWOCpnt) {
    var _a, _b;
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
    let cpntClassVar = getCpntClassVar(sourceFile);
    let insertPos = 0;
    let final = "";
    let apiCallArgument = ![null, undefined, ""].includes(options.apiCallClass) ?
        `new ${options.apiCallClass}(this.formsService.${lowerCamelCaseCpntClassName}.${options.name}.value)` :
        `this.formsService.${lowerCamelCaseCpntClassName}.${options.name}.value`;
    let snippet = `
    mainForm = new WMLForm({})
    formVars = ENV.${lowerCamelCaseCpntClassName}.${options.name}
_add_newline_here_
    submitBtn = new WmlButtonZeroParams({
      type:WMLButtonParamsTypeEnum.PRIMARY,
      text:new WMLUIProperty({
        text:"global.submiBtn"
      }),
      button:new WMLUIProperty({
        click:()=> this.clickSubmitBtn()
      })
    })
_add_newline_here_
    clickSubmitBtn = ()=>{
      this.baseService.submitForm(new BaseServiceSubmitFormParams({
        rootFormGroup:this.formsService.${lowerCamelCaseCpntClassName}.${options.name},
        cdref:this.cdref,
        validFormPredicateTypeDefault:{
          ngUnsub:this.ngUnsub,
          // @ts-ignore
          apiCall$:${options.apiCall}(${apiCallArgument})
        }
      }))
    }
_add_newline_here_
    createTextField =()=>{
      let val = {
        base:ENV.${lowerCamelCaseCpntClassName}.${options.name}.${(_a = options.fields[0]) !== null && _a !== void 0 ? _a : "name"}FormControlName,
        get field(){
          return this.base +"Field"
        }
      }
      let field = this.baseService.createInputFormField({
        id:this.idPrefix(val.field),
        labelValue:"${cpntClassNameWOCpnt}.${options.name}."+val.field+".label",
        fieldFormControlName:this.formVars[val.base+"FormControlName"],
        fieldParentForm:this.formsService.${lowerCamelCaseCpntClassName}.${options.name},
        selfType:"standalone",
        errorMsgs:createI18nErrorMsgsBasedOnValidations(
          "${cpntClassNameWOCpnt}.${options.name}",val.field,["required","min","digit"]
        ),
      })
      field.deleteRequiredLabelPart()
      return field
    }
_add_newline_here_
    initForm = ()=>{
      let fields = [
        this.createTextField(),
      ]
      this.mainForm.updateFields(fields)
    }
  `;
    let text = sourceFile.getFullText();
    let indentation = (0, utils_1.getIndentation)(text);
    if (!lastProperty) {
        let constructor = (0, utils_1.wmlFindChildNode)(cpntClassVar.members, ts.SyntaxKind.Constructor, 
        // @ts-ignore
        // (prop:ts.ClassElement)=>prop.name?.getText().includes("constructor"),
        "something is wrong with the components constructor", true, false);
        if (!constructor) {
            let openBrace = (0, utils_1.wmlFindChildNode)(cpntClassVar, ts.SyntaxKind.OpenBraceToken, "its with classes open brace token is it missing");
            insertPos = openBrace === null || openBrace === void 0 ? void 0 : openBrace.end;
        }
        else {
            insertPos = constructor === null || constructor === void 0 ? void 0 : constructor.end;
        }
        snippet = (0, utils_1.removeLeadingWhitespace)(snippet, "\t\t\t\t\n");
    }
    else {
        insertPos = lastProperty.end;
        snippet = (0, utils_1.removeLeadingWhitespace)(snippet, lastProperty.getFullText());
    }
    final = `${(_b = indentation[0]) !== null && _b !== void 0 ? _b : ' '}${snippet}`;
    let changes = [new change_1.InsertChange(targetFilePath, insertPos, final)];
    (0, updateFile_1.wmlUpdateFile)(tree, targetFilePath, changes);
}
function getCpntClassVar(sourceFile) {
    return sourceFile.statements
        .filter(ts.isClassDeclaration)
        .find((v) => {
        var _a;
        return (_a = v.name) === null || _a === void 0 ? void 0 : _a.getText().endsWith("Component");
    });
}
//# sourceMappingURL=index.js.map