"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateServiceMethodTemplate = void 0;
const schematics_1 = require("@angular-devkit/schematics");
const core_1 = require("@angular-devkit/core");
const ts = require("@schematics/angular/third_party/github.com/Microsoft/TypeScript/lib/typescript");
const change_1 = require("@schematics/angular/utility/change");
const updateFile_1 = require("../utils/updateFile");
const ast_utils_1 = require("@schematics/angular/utility/ast-utils");
const utils_1 = require("../utils/utils");
class TemplateServiceMethodSchema {
    constructor() {
        this.envFilePath = utils_1.DevEnvFile;
    }
}
function generateServiceMethodTemplate(options) {
    return () => {
        let serviceMethodFilePath = (0, core_1.normalize)(`/${options.path}`);
        const templateSource = (0, schematics_1.apply)((0, schematics_1.url)('./files'), [
            (0, schematics_1.applyTemplates)(Object.assign({ capitalize: schematics_1.strings.capitalize }, options)),
            (0, schematics_1.move)(serviceMethodFilePath),
        ]);
        return (0, schematics_1.chain)([
            (0, schematics_1.mergeWith)(templateSource),
            updateServiceFile(options)
        ]);
    };
}
exports.generateServiceMethodTemplate = generateServiceMethodTemplate;
let updateServiceFile = (options) => {
    return (tree) => {
        var _a;
        let targetFileName = options.serviceName + ".service.ts";
        let targetFilePath = (0, core_1.normalize)(`${options.path}/${targetFileName}`);
        let targetFile = tree.read(targetFilePath);
        if (!targetFile) {
            throw new schematics_1.SchematicsException(`${targetFilePath} not found make sure the command is run in the directory of the service file`);
        }
        let sourceFile = ts.createSourceFile(targetFilePath, targetFile.toString(), ts.ScriptTarget.Latest, true);
        let serviceClassName = options.serviceName + "Service";
        let classifiedServiceClassName = schematics_1.strings.classify(serviceClassName);
        let serviceClassVar = sourceFile.statements
            .filter(ts.isClassDeclaration)
            .find((v) => {
            var _a;
            return ((_a = v.name) === null || _a === void 0 ? void 0 : _a.getText()) === classifiedServiceClassName;
        });
        let capitalizedMethodName = schematics_1.strings.capitalize(options.name);
        addServiceMethodToServiceClass(options, capitalizedMethodName, serviceClassName, serviceClassVar, targetFilePath, tree);
        addNeededImportsToServiceClass(sourceFile, capitalizedMethodName, options, targetFilePath, tree);
        try {
            let { targetPath, targetFile } = (0, utils_1.getFile)(tree, utils_1.DevEnvFile);
            let sourceFile = ts.createSourceFile(targetPath, targetFile.toString(), ts.ScriptTarget.Latest, true);
            let devEnv = (_a = sourceFile === null || sourceFile === void 0 ? void 0 : sourceFile.statements) === null || _a === void 0 ? void 0 : _a.filter(ts.isClassDeclaration).find((v) => {
                var _a;
                return ((_a = v.name) === null || _a === void 0 ? void 0 : _a.getText()) === 'DevEnv';
            });
            let serviceClassProp = devEnv.members.find((prop) => {
                var _a;
                return ((_a = prop.name) === null || _a === void 0 ? void 0 : _a.getText()) === serviceClassName;
            });
            let serviceClassObj = (0, ast_utils_1.findNodes)(serviceClassProp, ts.SyntaxKind.ObjectLiteralExpression, 1)[0];
            let occurrencesCount = serviceClassObj.properties.length;
            let text = serviceClassObj.getFullText();
            let insertPos = serviceClassObj.properties.pos;
            let methodString = `${options.name}:new WMLEndpoint({
      url:()=> this.backendDomain0 + "${options.apiRoute}",
    })`;
            let finalVar = methodString;
            if (occurrencesCount > 0) {
                let lastEndpointLiteral = [
                    ...serviceClassObj.properties,
                ].pop();
                let indentation = (0, utils_1.getIndentation)(text);
                let endpointText = `${indentation[0] || ' '}${methodString}`;
                insertPos = lastEndpointLiteral.end;
                finalVar = `,${endpointText}`;
            }
            let changes = [new change_1.InsertChange(targetPath, insertPos, finalVar)];
            (0, updateFile_1.wmlUpdateFile)(tree, targetPath, changes);
        }
        catch (e) {
        }
        return tree;
    };
};
function addNeededImportsToServiceClass(sourceFile, capitalizedMethodName, options, targetFilePath, host) {
    let importStatements = sourceFile.statements
        .filter(ts.isImportDeclaration);
    let occurences = importStatements.length;
    let text = sourceFile.getFullText();
    let insertPos = 0;
    let final = `import { ${capitalizedMethodName}UIRequestBody, ${capitalizedMethodName}UIResponseBody, ${options.name}Load, ${options.name}Success } from './${options.name}';`;
    if (occurences > 0) {
        let lastMember = [
            ...importStatements
        ].pop();
        let indentation = (0, utils_1.getIndentation)(text);
        final = `${indentation[0] || ' '}${final}`;
        insertPos = lastMember.end;
    }
    let changes = [new change_1.InsertChange(targetFilePath, insertPos, final)];
    (0, updateFile_1.wmlUpdateFile)(host, targetFilePath, changes);
}
function addServiceMethodToServiceClass(options, capitalizedMethodName, serviceClassName, serviceClassVar, targetFilePath, host) {
    let serviceMethodString = `  ${options.name} = (uiBody = new ${capitalizedMethodName}UIRequestBody(),raw =false)=>{
    return iif(
      ()=>ENV.${serviceClassName}.${options.name}.automate,
      of(new ${capitalizedMethodName}UIResponseBody()),
      ${options.name}Load(uiBody)
        .pipe(
          concatMap((apiBody)=>{
            return this.http
              .post(ENV.${serviceClassName}.${options.name}.url(),apiBody)
              .pipe(raw ? tap() : map(${options.name}Success))
          })
        )
      )
  }`;
    let occurences = serviceClassVar.members.length;
    let text = serviceClassVar.getFullText();
    let insertPos = serviceClassVar.members.pos;
    let final = serviceMethodString;
    if (occurences > 0) {
        let lastMember = [
            ...serviceClassVar.members
        ].pop();
        let indentation = text.match(/\r?\n(\r?)\s*/) || [];
        final = `${indentation[0] || ' '}${serviceMethodString}`;
        insertPos = lastMember.end;
    }
    let changes = [new change_1.InsertChange(targetFilePath, insertPos, final)];
    (0, updateFile_1.wmlUpdateFile)(host, targetFilePath, changes);
}
//# sourceMappingURL=index.js.map