"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FormsServiceFilePath = exports.DevEnvFile = exports.getFile = exports.wmlFindChildNode = exports.prepareForUpdate = exports.addImportStatement = exports.getTsMemberInfo = exports.wmlMsgs = exports.getIndentation = exports.wmlCreateSourceFile = exports.removeLeadingWhitespace = void 0;
const ts = require("@schematics/angular/third_party/github.com/Microsoft/TypeScript/lib/typescript");
const schematics_1 = require("@angular-devkit/schematics");
const updateFile_1 = require("./updateFile");
const change_1 = require("@schematics/angular/utility/change");
const core_1 = require("@angular-devkit/core");
function removeLeadingWhitespace(inputString, prefixString, allowFirstLineUnshift = true) {
    var _a, _b, _c, _d;
    const lines = inputString.split('\n');
    if (allowFirstLineUnshift) {
        lines.shift();
    }
    if (lines.length === 0) {
        return inputString; // Nothing to process
    }
    let firstLineIndentation = (_b = (_a = RegExp(/^\s*/).exec(lines[0])) === null || _a === void 0 ? void 0 : _a[0]) !== null && _b !== void 0 ? _b : '';
    let prefixLineIndentation = (_d = (_c = RegExp(/^\s*/).exec(prefixString)) === null || _c === void 0 ? void 0 : _c[0]) !== null && _d !== void 0 ? _d : '';
    let result = lines.map(line => {
        let newLine = line
            .replace(firstLineIndentation, prefixLineIndentation)
            .replace(/_add_newline_here_/g, `\n${firstLineIndentation}`);
        return newLine;
    })
        .join('');
    return result;
}
exports.removeLeadingWhitespace = removeLeadingWhitespace;
function wmlCreateSourceFile(targetFilePath, tree) {
    let targetFile = tree.read(targetFilePath);
    if (!targetFile) {
        throw new schematics_1.SchematicsException(`This may be an internal issue but so far
    the ${targetFilePath} is not found so I cant selct the file to edit `);
    }
    return ts.createSourceFile(targetFilePath, targetFile.toString(), ts.ScriptTarget.Latest, true);
}
exports.wmlCreateSourceFile = wmlCreateSourceFile;
let getIndentation = (text = "") => {
    var _a;
    return (_a = RegExp(/\r?\n(\r?)\s*/).exec(text)) !== null && _a !== void 0 ? _a : [];
};
exports.getIndentation = getIndentation;
exports.wmlMsgs = {
    syntaxIssue: (addtl) => "I may have an internal error but I see that the syntax for this component class is incoorect  Fix the issue, " + addtl
};
function getTsMemberInfo(x) {
    console.log(x.getText(), x.kind);
}
exports.getTsMemberInfo = getTsMemberInfo;
let addImportStatement = (targetFilePath, tree, importStatementSnippet, packageName, importStatement, importMembers) => {
    let sourceFile = wmlCreateSourceFile(targetFilePath, tree);
    let statements = sourceFile.statements;
    let stmnt = statements.find((x) => {
        return x.getFullText().includes(packageName);
    });
    if (!stmnt) {
        importStatementSnippet += importStatement;
    }
    else {
        let importClause = wmlFindChildNode(stmnt, ts.SyntaxKind.ImportClause, "its with the open brace of the import statement related to " + packageName);
        let namedImport = wmlFindChildNode(importClause, ts.SyntaxKind.NamedImports, "its with the open brace of the import statement related to " + packageName);
        let openBrace = wmlFindChildNode(namedImport, ts.SyntaxKind.OpenBraceToken, "its with the open brace of the import statement related to " + packageName, true);
        // SyntaxList
        let syntaxLists = namedImport.getChildren().filter((x) => {
            return x.kind === ts.SyntaxKind.SyntaxList;
        })
            .map((x) => {
            return x.getFullText().trim();
        });
        let partialString = importMembers.filter((x) => {
            return !syntaxLists.includes(x.trim());
        }).join(",") + " ,";
        prepareForUpdate(partialString, targetFilePath, tree, openBrace.end, "");
    }
    return importStatementSnippet;
};
exports.addImportStatement = addImportStatement;
function prepareForUpdate(snippet, targetFilePath, tree, insertPos = 0, indentTarget = "", spacingPrefix = "") {
    var _a;
    let final = "";
    let indentation = (0, exports.getIndentation)(indentTarget);
    snippet = removeLeadingWhitespace(snippet, indentTarget);
    final = `${spacingPrefix}${(_a = indentation[0]) !== null && _a !== void 0 ? _a : ' '}${snippet}`;
    let changes = [new change_1.InsertChange(targetFilePath, insertPos, final)];
    (0, updateFile_1.wmlUpdateFile)(tree, targetFilePath, changes);
}
exports.prepareForUpdate = prepareForUpdate;
function wmlFindChildNode(parent, syntaxKind, issueMsg, printChildInfo = false, throwIfNotFound = true) {
    var _a, _b;
    // @ts-ignore
    let node = ((_b = (_a = parent === null || parent === void 0 ? void 0 : parent.getChildren) === null || _a === void 0 ? void 0 : _a.call(parent)) !== null && _b !== void 0 ? _b : parent)
        .find(((x) => {
        if (printChildInfo) {
            getTsMemberInfo(x);
        }
        if (syntaxKind instanceof Function) {
            return syntaxKind(x);
        }
        else {
            return x.kind === syntaxKind;
        }
    }));
    if (!node && throwIfNotFound) {
        throw new schematics_1.SchematicsException(exports.wmlMsgs.syntaxIssue(issueMsg));
    }
    return node;
}
exports.wmlFindChildNode = wmlFindChildNode;
function getFile(tree, path) {
    let targetPath = (0, core_1.normalize)(path);
    let targetFile = tree.read(targetPath);
    if (!targetFile) {
        throw new schematics_1.SchematicsException(`${targetPath} not found`);
    }
    return { targetPath, targetFile };
}
exports.getFile = getFile;
exports.DevEnvFile = "src/environments/environment.dev.ts";
exports.FormsServiceFilePath = "src/app/shared/services/forms/forms.service.ts";
//# sourceMappingURL=utils.js.map